package xyz.lychee.lagfixer.libs.adventure.util;

import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Optional;
import java.util.ServiceConfigurationError;
import java.util.ServiceLoader;
import java.util.Set;
import xyz.lychee.lagfixer.libs.adventure.internal.properties.AdventureProperties;
import xyz.lychee.lagfixer.libs.annotations.NotNull;

public final class Services {
   private static final boolean SERVICE_LOAD_FAILURES_ARE_FATAL;

   private Services() {
   }

   public static <P> @NotNull Optional<P> service(final @NotNull Class<P> type) {
      ServiceLoader<P> loader = Services0.loader(type);
      Iterator<P> it = loader.iterator();

      while(true) {
         if (it.hasNext()) {
            Object instance;
            try {
               instance = it.next();
            } catch (Throwable var5) {
               Throwable t = var5;
               if (!SERVICE_LOAD_FAILURES_ARE_FATAL) {
                  continue;
               }

               throw new IllegalStateException("Encountered an exception loading service " + type, t);
            }

            if (it.hasNext()) {
               throw new IllegalStateException("Expected to find one service " + type + ", found multiple");
            }

            return Optional.of(instance);
         }

         return Optional.empty();
      }
   }

   public static <P> @NotNull Optional<P> serviceWithFallback(final @NotNull Class<P> type) {
      ServiceLoader<P> loader = Services0.loader(type);
      Iterator<P> it = loader.iterator();
      P firstFallback = null;

      while(it.hasNext()) {
         Object instance;
         try {
            instance = it.next();
         } catch (Throwable var6) {
            Throwable t = var6;
            if (!SERVICE_LOAD_FAILURES_ARE_FATAL) {
               continue;
            }

            throw new IllegalStateException("Encountered an exception loading service " + type, t);
         }

         if (!(instance instanceof Fallback)) {
            return Optional.of(instance);
         }

         if (firstFallback == null) {
            firstFallback = instance;
         }
      }

      return Optional.ofNullable(firstFallback);
   }

   public static <P> Set<P> services(final Class<? extends P> clazz) {
      ServiceLoader<? extends P> loader = Services0.loader(clazz);
      Set<P> providers = new HashSet();
      Iterator<? extends P> it = loader.iterator();

      while(it.hasNext()) {
         Object instance;
         try {
            instance = it.next();
         } catch (ServiceConfigurationError var6) {
            ServiceConfigurationError ex = var6;
            if (!SERVICE_LOAD_FAILURES_ARE_FATAL) {
               continue;
            }

            throw new IllegalStateException("Encountered an exception loading a provider for " + clazz + ": ", ex);
         }

         providers.add(instance);
      }

      return Collections.unmodifiableSet(providers);
   }

   static {
      SERVICE_LOAD_FAILURES_ARE_FATAL = Boolean.TRUE.equals(AdventureProperties.SERVICE_LOAD_FAILURES_ARE_FATAL.value());
   }

   public interface Fallback {
   }
}
